@@ -1,9 +1,9 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2004-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2004-2008, 2010, 2012 J. David Ibáñez <jdavid.ibp@gmail.com>
 # Copyright (C) 2005 Piotr Macuk <piotr@macuk.pl>
-# Copyright (C) 2006-2007 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2007 Henry Obein <henry@itaapy.com>
-# Copyright (C) 2007 Nicolas Deram <nicolas@itaapy.com>
+# Copyright (C) 2006-2007, 2009-2010 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2007 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2007 Nicolas Deram <nderam@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,15 +19,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.core import merge_dicts
-from itools.datatypes import String, Integer
+from itools.datatypes import String, Unicode
 from itools.handlers import TextFile, guess_encoding, register_handler_class
-from itools.xapian import make_catalog, CatalogAware
 from parser import parse
 
 
 
-class Row(list, CatalogAware):
+# TODO Drop the 'Row' class, use a list or a tuple instead.
+# Row.get_value(name) will be replaced by CSVFile.get_value(row, name)
+class Row(list):
 
     def get_value(self, name):
         if self.columns is None:
@@ -42,17 +42,9 @@ class Row(list, CatalogAware):
         clone = self.__class__(self)
         clone.number = self.number
         clone.columns = self.columns
-
         return clone
 
 
-    def get_catalog_values(self):
-        values = {'__number__': self.number}
-        for column, name in enumerate(self.columns):
-            values[name] = self[column]
-        return values
-
-
 
 class CSVFile(TextFile):
 
@@ -61,47 +53,25 @@ class CSVFile(TextFile):
 
     # Hash with column names and its types
     # Example: {'firstname': Unicode, 'lastname': Unicode, 'age': Integer}
-    # To index some columns the schema should be declared as:
-    # schema = {'firstname': Unicode, 'lastname': Unicode,
-    #           'age': Integer(is_indexed=True)}
     schema = None
 
     # List of the schema column names
     # Example: ['firstname', 'lastname', 'age']
     columns = None
 
-    # The class to use for each row (this allows easy specialization)
-    # TODO The ability to change the row class should be removed.
-    row_class = Row
-
     # Parsing options
     class_csv_guess = False
+    has_header = False
     skip_header = False
 
 
     #########################################################################
     # Load & Save
     #########################################################################
-    clone_exclude = TextFile.clone_exclude | frozenset(['catalog'])
-
-
     def reset(self):
+        self.header = None
         self.lines = []
         self.n_lines = 0
-        # Initialize the catalog if needed (Index&Search)
-        # (we look if we have at least one indexed field in schema/columns)
-        self.catalog = None
-        schema = self.schema
-        if schema is not None:
-            for name in self.columns:
-                field_cls = schema[name]
-                if getattr(field_cls, 'is_indexed', False):
-                    fields = merge_dicts(schema,
-                                         __number__=Integer(is_key_field=True,
-                                                            is_stored=True,
-                                                            is_indexed=True))
-                    self.catalog = make_catalog(None, fields)
-                    break
 
 
     def new(self):
@@ -109,21 +79,34 @@ class CSVFile(TextFile):
 
 
     def _load_state_from_file(self, file):
-        # Read the data, and find out the encoding
+        # Guess encoding
         data = file.read()
         self.encoding = guess_encoding(data)
 
-        schema = self.schema
-        columns = self.columns
+        # Build the parser
+        parser = parse(data, self.columns, self.schema, self.class_csv_guess,
+                       self.has_header, self.encoding)
 
-        for line in parse(data, columns, schema, guess=self.class_csv_guess,
-                          skip_header=self.skip_header,
-                          encoding=self.encoding):
+        # Header
+        if self.has_header:
+            self.header = parser.next()
+
+        # Content
+        for line in parser:
             self._add_row(line)
 
 
-    def to_str(self, encoding='UTF-8', separator=','):
+    def to_str(self, encoding='UTF-8', separator=',', newline='\n'):
+        def escape(data):
+            return '"%s"' % data.replace('"', '""')
+
         lines = []
+        # Header
+        if self.has_header:
+            line = [ escape(Unicode.encode(x, encoding)) for x in self.header ]
+            line = separator.join(line)
+            lines.append(line)
+
         # When schema or columns (or both) are None there is plain
         # string to string conversion
         schema = self.schema
@@ -137,13 +120,13 @@ class CSVFile(TextFile):
                         data = datatype.encode(row[i], encoding=encoding)
                     except TypeError:
                         data = datatype.encode(row[i])
-                    line.append('"%s"' % data.replace('"', '""'))
+                    line.append(escape(data))
                 lines.append(separator.join(line))
         else:
             for row in self.get_rows():
-                line = [ '"%s"' % x.replace('"', '""') for x in row ]
+                line = [ escape(x) for x in row ]
                 lines.append(separator.join(line))
-        return '\n'.join(lines)
+        return newline.join(lines)
 
 
     #########################################################################
@@ -153,15 +136,12 @@ class CSVFile(TextFile):
         """Append new row as an instance of row class.
         """
         # Build the row
-        row = self.row_class(row)
+        row = Row(row)
         row.number = self.n_lines
         row.columns = self.columns
         # Add
         self.lines.append(row)
         self.n_lines += 1
-        # Index
-        if self.catalog is not None:
-            self.catalog.index_document(row)
 
         return row
 
@@ -176,12 +156,6 @@ class CSVFile(TextFile):
     #########################################################################
     # API / Public
     #########################################################################
-    def is_indexed(self):
-        """Check if at least one index is available for searching, etc.
-        """
-        return self.catalog is not None
-
-
     def get_nrows(self):
         return len([ x for x in self.lines if x is not None])
 
@@ -223,9 +197,6 @@ class CSVFile(TextFile):
     def update_row(self, index, **kw):
         row = self.get_row(index)
         self.set_changed()
-        # Un-index
-        if self.catalog is not None:
-            self.catalog.unindex_document(row.number)
         # Update
         columns = self.columns
         if columns is None:
@@ -236,9 +207,6 @@ class CSVFile(TextFile):
             for name in kw:
                 column = columns.index(name)
                 row[column] = kw[name]
-        # Index
-        if self.catalog is not None:
-            self.catalog.index_document(row)
 
 
     def del_row(self, number):
@@ -252,10 +220,6 @@ class CSVFile(TextFile):
             raise IndexError, 'list assignment index out of range'
         self.lines[number] = None
 
-        # Unindex
-        if self.catalog is not None:
-            self.catalog.unindex_document(row.number)
-
 
     def del_rows(self, numbers):
         """Delete rows at the given line numbers. Count begins at 0.
@@ -266,17 +230,6 @@ class CSVFile(TextFile):
             self.del_row(i)
 
 
-    def search(self, query=None, **kw):
-        """Return list of row numbers returned by executing the query.
-        """
-        if self.catalog is None:
-            raise IndexError, 'no index is defined in the schema'
-
-        result = self.catalog.search(query, **kw)
-        return [ doc.__number__
-                 for doc in result.get_documents(sort_by='__number__') ]
-
-
     def get_unique_values(self, name):
         return set([ x.get_value(name) for x in self.get_rows() ])
 
