@@ -1,8 +1,11 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2008 Nicolas Deram <nicolas@itaapy.com>
-# Copyright (C) 2007 Henry Obein <henry@itaapy.com>
-# Copyright (C) 2007-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2007-2008 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2005-2008 Nicolas Deram <nderam@gmail.com>
+# Copyright (C) 2007-2008 Sylvain Taverne <taverne.sylvain@gmail.com>
+# Copyright (C) 2007-2008, 2010 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2007-2012 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2008-2011 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2009 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2010 Alexis Huet <alexis@itaapy.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,14 +21,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
+from copy import deepcopy
 from datetime import datetime
 
 # Import from itools
-from itools.core import merge_dicts
-from itools.datatypes import DateTime, String, Integer, Unicode
+from itools.datatypes import DateTime, String, Unicode
 from itools.handlers import File
-from itools.xapian import make_catalog
-from itools.xapian import PhraseQuery, CatalogAware
 from csv_ import CSVFile
 from parser import parse
 
@@ -33,20 +34,30 @@ from parser import parse
 ###########################################################################
 # Parser
 ###########################################################################
-def unescape_data(data):
+escape_table = (
+    ('\r', r'\r'),
+    ('\n', r'\n'))
+
+
+def unescape_data(data, escape_table=escape_table):
     """Unescape the data
     """
-    data = [ x.replace("\\r", "\r").replace("\\n", "\n")
-             for x in data.split('\\\\') ]
-    return '\\'.join(data)
+    out = []
+    for segment in data.split(r"\\"):
+        for c, c_escaped in escape_table:
+            segment = segment.replace(c_escaped, c)
+        out.append(segment)
+
+    return '\\'.join(out)
 
 
 
-def escape_data(data):
+def escape_data(data, escape_table=escape_table):
     """Escape the data
     """
-    data = data.replace("\\", "\\\\")
-    data = data.replace("\r", "\\r").replace("\n", "\\n")
+    data = data.replace("\\", r"\\")
+    for c, c_escaped in escape_table:
+        data = data.replace(c, c_escaped)
     return data
 
 
@@ -97,7 +108,10 @@ def fold_line(data):
 
 
 
-def read_name(line):
+# XXX The RFC only allows '-', we allow more because this is used by
+# itools.database (metadata). This set matches checkid (itools.handlers)
+allowed = frozenset(['-', '_', '.', '@'])
+def read_name(line, allowed=allowed):
     """Reads the property name from the line. Returns the name and the
     rest of the line:
 
@@ -116,7 +130,7 @@ def read_name(line):
         c = line[idx]
         if c in (';', ':'):
             return line[:idx], line[idx:]
-        if c.isalnum() or c in ('-', '_'):
+        if c.isalnum() or c in allowed:
             idx += 1
             continue
         raise SyntaxError, "unexpected character '%s' (%s)" % (c, ord(c))
@@ -141,12 +155,11 @@ def read_name(line):
 #     8 --> value begun
 
 error1 = 'unexpected character (%s) at status %s'
-error2 = 'unexpected repeated character (%s) at status %s'
 
 def get_tokens(property):
     parameters = {}
     value = ''
-    status, last = 0, ''
+    status = 0
 
     # Init status
     c, property = property[0], property[1:]
@@ -169,7 +182,7 @@ def get_tokens(property):
 
         # param-name begun
         elif status == 2:
-            if c.isalnum() or c in ('-'):
+            if c.isalnum() or c in ('-', '_'):
                 param_name += c
             elif c == '=':
                 parameters[param_name] = []
@@ -197,13 +210,17 @@ def get_tokens(property):
         # param-value quoted begun (just after '"')
         elif status == 4:
             if c == '"':
-                if last == '"':
-                    raise SyntaxError, error2 % (c, status)
-                last = '"'
                 status = 6
+            elif c == "\\":
+                param_value += c
+                status = 41
             else:
                 param_value += c
 
+        elif status == 41: # XXX Special case (not for ical)
+            param_value += c
+            status = 4
+
         # param-value NOT quoted begun
         elif status == 5:
             if c == ':':
@@ -258,7 +275,6 @@ def parse_table(data):
     are byte strings.
     """
     for line in unfold_lines(data):
-        parameters = {}
         name, line = read_name(line)
         # Read the parameters and the property value
         value, parameters = get_tokens(line)
@@ -270,16 +286,21 @@ def parse_table(data):
 # Helper functions
 ###########################################################################
 def is_multilingual(datatype):
-    return issubclass(datatype, Unicode) and datatype.multiple
+    return getattr(datatype, 'multilingual', False)
 
 
+def is_multiple(datatype):
+    return getattr(datatype, 'multiple', False)
+
 
 def deserialize_parameters(parameters, schema, default=String(multiple=True)):
     for name in parameters:
-        value = parameters[name]
         datatype = schema.get(name, default)
+        if datatype is None:
+            raise ValueError, 'parameter "{0}" not defined'.format(name)
         # Decode
-        value = [ datatype.decode(x) for x in value ]
+        value = parameters[name]
+        value = [ decode_param_value(x, datatype) for x in value ]
         # Multiple or single
         if not datatype.multiple:
             if len(value) > 1:
@@ -306,7 +327,7 @@ class UniqueError(ValueError):
     def __str__(self):
         return (
             u'the "{field}" field must be unique, the "{value}" value is '
-            u' already used.')
+            u' already used.').format(field=self.name, value=self.value)
 
 
 ###########################################################################
@@ -327,6 +348,17 @@ class Property(object):
         self.parameters = kw or None
 
 
+    def clone(self):
+        # Copy the value and parameters
+        value = deepcopy(self.value)
+        parameters = {}
+        for p_key, p_value in self.parameters.iteritems():
+            c_value = deepcopy(p_value)
+            parameters[p_key] = c_value
+
+        return Property(value, **parameters)
+
+
     def get_parameter(self, name, default=None):
         if self.parameters is None:
             return default
@@ -339,10 +371,35 @@ class Property(object):
         self.parameters[name] = value
 
 
+    def __eq__(self, other):
+        if type(other) is not Property:
+            return False
+        if self.value != other.value:
+            return False
+        return self.parameters == other.parameters
+
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+
+
+params_escape_table = (
+    ('"', r'\"'),
+    ('\r', r'\r'),
+    ('\n', r'\n'))
+
 
 def encode_param_value(p_name, p_value, p_datatype):
     p_value = p_datatype.encode(p_value)
-    if '"' in p_value:
+
+    # Special case (not used by ical)
+    if getattr(p_datatype, 'escape', False):
+        p_value = escape_data(p_value, params_escape_table)
+        return '"%s"' % p_value
+
+    # Standard case (ical behavior)
+    if '"' in p_value or '\n' in p_value:
         error = 'the "%s" parameter contains a double quote'
         raise ValueError, error % p_name
     if ';' in p_value or ':' in p_value or ',' in p_value:
@@ -350,8 +407,15 @@ def encode_param_value(p_name, p_value, p_datatype):
     return p_value
 
 
+def decode_param_value(p_value, p_datatype):
+    # Special case (not used by ical)
+    if getattr(p_datatype, 'escape', False):
+        p_value = unescape_data(p_value, params_escape_table)
 
-def property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
+    return p_datatype.decode(p_value)
+
+
+def _property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
     """This method serializes the given property to a byte string:
 
       name[;parameters]=value
@@ -369,13 +433,15 @@ def property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
     parameters = []
     for p_name in p_names:
         p_value = property.parameters[p_name]
+        if p_value is None:
+            continue
         # Find out the datatype for the parameter
         p_datatype = p_schema.get(p_name)
         if not p_datatype:
             p_datatype = String(multiple=True)
         # Serialize the parameter
         # FIXME Use the encoding
-        if getattr(p_datatype, 'multiple', False):
+        if is_multiple(p_datatype):
             p_value = [
                 encode_param_value(p_name, x, p_datatype) for x in p_value ]
             p_value = ','.join(p_value)
@@ -389,6 +455,9 @@ def property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
         value = datatype.encode(property.value, encoding=encoding)
     else:
         value = datatype.encode(property.value)
+    if type(value) is not str:
+        raise ValueError, 'property "{0}" is not str but {1}'.format(
+                name, type(value))
     value = escape_data(value)
 
     # Ok
@@ -396,8 +465,16 @@ def property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
     return fold_line(property)
 
 
+def property_to_str(name, property, datatype, p_schema, encoding='utf-8'):
+    try:
+        return _property_to_str(name, property, datatype, p_schema, encoding)
+    except StandardError:
+        err = 'failed to serialize "%s" property, probably a bad value'
+        raise ValueError, err % name
 
-class Record(list, CatalogAware):
+
+
+class Record(dict):
 
     __slots__ = ['id', 'record_properties']
 
@@ -405,64 +482,44 @@ class Record(list, CatalogAware):
     def __init__(self, id, record_properties):
         self.id = id
         self.record_properties  = record_properties
-        self.append({})
 
 
     def __getattr__(self, name):
         if name == '__number__':
             return self.id
-        version = self[-1]
-        if name not in version:
+        if name not in self:
             raise AttributeError, "'%s' object has no attribute '%s'" % (
                 self.__class__.__name__, name)
 
-        property = version[name]
+        property = self[name]
         if type(property) is list:
             return [ x.value for x in property ]
         return property.value
 
 
     def get_property(self, name):
-        version = self[-1]
-        if name in version:
-            return version[name]
-
-        return None
+        return self.get(name)
 
 
     # For indexing purposes
     def get_value(self, name):
-        version = self[-1]
-        if name not in version:
+        property = self.get(name)
+        if property is None:
             return None
 
-        property = version[name]
         if type(property) is list:
             return [ x.value for x in property ]
         return property.value
 
 
-    def get_catalog_values(self):
-        values = {'__id__': self.id}
-        for name in self.record_properties.iterkeys():
-            values[name] = self.get_value(name)
-        return values
-
-
 
 class Table(File):
 
     record_class = Record
-    incremental_save = True
 
     #######################################################################
     # Hash with field names and its types
     # Example: {'firstname': Unicode, 'lastname': Unicode, 'age': Integer}
-    # To index some fields the schema should be declared as:
-    # record_properties = {
-    #     'firstname': Unicode,
-    #     'lastname': Unicode,
-    #     'age': Integer(is_indexed=True)}
     #######################################################################
     schema = {}
     record_properties = {}
@@ -489,9 +546,9 @@ class Table(File):
         return String(multiple=True)
 
 
-    def properties_to_dict(self, properties, version, first=False):
+    def properties_to_dict(self, properties, record, first=False):
         """Add the given "properties" as Property objects or Property objects
-        list to the given dictionnary "version".
+        list to the given dictionnary "record".
         """
         # The variable 'first' defines whether we are talking about the
         # table properties (True) or a about records (False).
@@ -508,92 +565,72 @@ class Table(File):
 
             # Transform values to properties
             if is_multilingual(datatype):
-                language = value.parameters['language']
-                version.setdefault(name, [])
-                version[name] = [
-                    x for x in version[name]
-                    if x.parameters['language'] != language ]
-                version[name].append(value)
+                if type(value) is not list:
+                    value = [ value ]
+                record.setdefault(name, [])
+                for p in value:
+                    language = p.parameters['language']
+                    record[name] = [
+                        x for x in record[name]
+                        if x.parameters['language'] != language ]
+                    record[name].append(p)
             elif datatype.multiple:
                 if type(value) is list:
-                    version[name] = [ to_property(x) for x in value ]
+                    record[name] = [ to_property(x) for x in value ]
                 else:
-                    version[name] = [to_property(value)]
+                    record[name] = [to_property(value)]
             else:
-                version[name] = to_property(value)
+                record[name] = to_property(value)
 
 
     #######################################################################
     # Handlers
     #######################################################################
-    clone_exclude = File.clone_exclude | frozenset(['catalog'])
-
-
     def reset(self):
         self.properties = None
         self.records = []
         self.changed_properties = False
-        self.added_records = []
-        self.removed_records = []
-        # The catalog (for index and search)
-        fields = merge_dicts(
-            self.record_properties,
-            __id__=Integer(is_key_field=True, is_stored=True, is_indexed=True))
-        self.catalog = make_catalog(None, fields)
 
 
     def new(self):
         # Add the properties record
         properties = self.record_class(-1, self.record_properties)
-        properties[0] = {'ts': Property(datetime.now())}
+        properties['ts'] = Property(datetime.now())
         self.properties = properties
 
 
     def _load_state_from_file(self, file):
         # Load the records
         records = self.records
-        properties = self.properties
         record_properties = self.record_properties
 
         n = 0
-        version = None
         for name, value, parameters in parse_table(file.read()):
             if name == 'id':
-                version = {}
-                # Identifier and Sequence (id)
                 uid, seq = value.split('/')
-                # Record
                 uid = int(uid)
-                if uid == -1:
-                    # Tale properties
-                    if properties is None:
-                        properties = self.record_class(uid, record_properties)
-                        self.properties = properties
-                    record = properties
-                elif uid >= n:
-                    # New record
-                    records.extend([None] * (uid - n))
+                # Build the new record
+                if seq == 'DELETED':
+                    record = None
+                else:
                     record = self.record_class(uid, record_properties)
+                # Table properties, or new record, or updated record
+                if uid == -1:
+                    get_datatype = self.get_datatype
+                    self.properties = record
+                elif uid >= n:
+                    get_datatype = self.get_record_datatype
+                    records.extend([None] * (uid - n))
                     records.append(record)
                     n = uid + 1
-                else:
-                    # Get the record
-                    record = records[uid]
-                # Version
-                if seq == 'DELETED':
-                    # Deleted
-                    if uid == -1:
-                        properties = None
-                    else:
-                        records[uid] = None
-                        record = None
-                else:
-                    record[0] = version
-                # Table or record schema
-                if uid == -1:
-                    get_datatype = self.get_datatype
                 else:
                     get_datatype = self.get_record_datatype
+                    records[uid] = record
+                # Continue
+                continue
+
+            # Skip deleted records
+            if record is None:
                 continue
 
             # Deserialize the parameters
@@ -603,21 +640,18 @@ class Table(File):
             datatype = get_datatype(name)
             value = datatype.decode(value)
             property = Property(value, **parameters)
-            if getattr(datatype, 'multiple', False) is True:
-                version.setdefault(name, []).append(property)
-            elif name in version:
-                raise ValueError, "property '%s' can occur only once" % name
+            if is_multilingual(datatype) or is_multiple(datatype):
+                record.setdefault(name, []).append(property)
+            elif name in record:
+                msg = "record %s: property '%s' can occur only once"
+                raise ValueError, msg % (uid, name)
             else:
-                version[name] = property
-        # Index the records
-        for record in records:
-            if record is not None:
-                self.catalog.index_document(record)
+                record[name] = property
 
 
-    def _version_to_str(self, id, version):
+    def _record_to_str(self, id, record):
         lines = ['id:%d/0\n' % id]
-        names = version.keys()
+        names = record.keys()
         names.sort()
         # Table or record schema
         if id == -1:
@@ -629,10 +663,10 @@ class Table(File):
         p_schema = self.record_parameters
         for name in names:
             datatype = get_datatype(name)
-            if getattr(datatype, 'multiple', False) is True:
-                properties = version[name]
+            if is_multilingual(datatype) or is_multiple(datatype):
+                properties = record[name]
             else:
-                properties = [version[name]]
+                properties = [record[name]]
             for property in properties:
                 if property.value is None:
                     continue
@@ -646,75 +680,22 @@ class Table(File):
 
     def to_str(self):
         lines = []
-        id = 0
         # Properties record
         if self.properties is not None:
-            version = self.properties[0]
-            version = self._version_to_str(-1, version)
-            lines.append(version)
+            record = self._record_to_str(-1, self.properties)
+            lines.append(record)
         # Common record
+        id = 0
         for record in self.records:
             if record is not None:
-                version = record[0]
-                version = self._version_to_str(id, version)
-                lines.append(version)
+                record = self._record_to_str(id, record)
+                lines.append(record)
             # Next
             id += 1
 
         return ''.join(lines)
 
 
-    #######################################################################
-    # Save (use append for scalability)
-    #######################################################################
-    def _save_state(self):
-        if self.incremental_save is False:
-            File._save_state(self)
-            self.incremental_save = True
-            return
-
-        # Case 1: new file
-        if self.timestamp is None:
-            File._save_state(self)
-            self.changed_properties = False
-            self.added_records = []
-            self.removed_records = []
-            return
-
-        # Case 2: changed file (incremental save)
-        file = self.database.fs.open(self.key, 'a')
-        try:
-            # Added properties records
-            if self.changed_properties:
-                version = self.properties[0]
-                version = self._version_to_str(-1, version)
-                file.write(version)
-            self.changed_properties = False
-            # Added records
-            for id in self.added_records:
-                version = self.records[id][0]
-                version = self._version_to_str(id, version)
-                file.write(version)
-            self.added_records = []
-            # Removed records
-            for id, ts in self.removed_records:
-                file.write('id:%s/DELETED\n' % id)
-                file.write('ts:%s\n' % DateTime.encode(ts))
-                file.write('\n')
-            self.removed_records = []
-        finally:
-            file.close()
-
-
-    def save_state_to(self, key):
-        # TODO: this is a hack, for 0.50 this case should be covered by the
-        # handler's protocol
-        File.save_state_to(self, key)
-        if key == self.key:
-            self.added_records = []
-            self.removed_records = []
-
-
     #######################################################################
     # API / Public
     #######################################################################
@@ -730,19 +711,16 @@ class Table(File):
         for name in kw:
             datatype = self.get_record_datatype(name)
             if getattr(datatype, 'unique', False) is True:
-                if len(self.search(PhraseQuery(name, kw[name]))) > 0:
+                if self.search(name, kw[name]):
                     raise UniqueError(name, kw[name])
-        # Add version to record
+        # Make new record
         id = len(self.records)
         record = self.record_class(id, self.record_properties)
-        version = record[0]
-        self.properties_to_dict(kw, version)
-        version['ts'] = Property(datetime.now())
+        self.properties_to_dict(kw, record)
+        record['ts'] = Property(datetime.now())
         # Change
         self.set_changed()
-        self.added_records.append(id)
         self.records.append(record)
-        self.catalog.index_document(record)
         # Back
         return record
 
@@ -756,19 +734,13 @@ class Table(File):
         for name in kw:
             datatype = self.get_record_datatype(name)
             if getattr(datatype, 'unique', False) is True:
-                search = self.search(PhraseQuery(name, kw[name]))
+                search = self.search(name, kw[name])
                 if search and (search[0] != self.records[id]):
                     raise UniqueError(name, kw[name])
-        # Version of record
-        version = record[0]
-        self.properties_to_dict(kw, version)
-        version['ts'] = Property(datetime.now())
-        # Change
+        # Update record
         self.set_changed()
-        self.catalog.unindex_document(record.id)
-        self.added_records.append(id)
-        # Index
-        self.catalog.index_document(record)
+        self.properties_to_dict(kw, record)
+        record['ts'] = Property(datetime.now())
 
 
     def update_properties(self, **kw):
@@ -779,9 +751,8 @@ class Table(File):
             record = self.record_class(-1, self.record_properties)
             self.properties = record
 
-        version = record[0]
-        self.properties_to_dict(kw, version, first=True)
-        version['ts'] = Property(datetime.now())
+        self.properties_to_dict(kw, record, first=True)
+        record['ts'] = Property(datetime.now())
         # Change
         self.set_changed()
         self.changed_properties = True
@@ -794,10 +765,6 @@ class Table(File):
             raise LookupError, msg % id
         # Change
         self.set_changed()
-        if id not in self.added_records:
-            self.removed_records.append((id, datetime.now()))
-        self.added_records = [ x for x in self.added_records if x != id ]
-        self.catalog.unindex_document(record.id)
         self.records[id] = None
 
 
@@ -816,8 +783,7 @@ class Table(File):
 
 
     def get_records(self):
-        for id in self.get_record_ids():
-            yield self.get_record(id)
+        return ( x for x in self.records if x )
 
 
     def get_record_value(self, record, name, language=None):
@@ -862,9 +828,11 @@ class Table(File):
         if datatype.multiple:
             # Default
             if property is None:
-                # FIXME Probably we should check whether the datatype defines
-                # a default value.
-                return []
+                default = datatype.get_default()
+                # FIXME raise a TypeError instead
+                if type(default) is not list:
+                    return []
+                return default
             # Hit
             return [ x.value for x in property ]
 
@@ -892,21 +860,18 @@ class Table(File):
         except AttributeError:
             if name in self.record_properties:
                 datatype = self.get_datatype(name)
-                if getattr(datatype, 'multiple', False) is True:
+                if is_multilingual(datatype) or is_multiple(datatype):
                     return []
                 else:
                     return getattr(datatype, 'default')
 
 
-    def search(self, query=None, **kw):
-        """Return list of row numbers returned by executing the query.
-        """
-        result = self.catalog.search(query, **kw)
-        ids = [ doc.__id__ for doc in result.get_documents(sort_by='__id__') ]
-        return [ self.records[x] for x in ids ]
+    def search(self, key, value):
+        get = self.get_record_value
+        return [ x for x in self.records if x and get(x, key) == value ]
 
 
-    def update_from_csv(self, data, columns):
+    def update_from_csv(self, data, columns, skip_header=False):
         """Update the table by adding record from data
         The input parameters are :
 
@@ -915,7 +880,8 @@ class Table(File):
           columns and the table schema.
         """
         record_properties = self.record_properties
-        for line in parse(data, columns, record_properties):
+        for line in parse(data, columns, record_properties,
+                skip_header=skip_header):
             record = {}
             for index, key in enumerate(columns):
                 if key in record_properties:
