@@ -1,8 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2004-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2006-2008 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2007 David Versmisse <david.versmisse@itaapy.com>
-# Copyright (C) 2007 Henry Obein <henry@itaapy.com>
+# Copyright (C) 2004-2012 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2006-2008, 2010 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2007, 2009 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2007, 2011 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2008 Gautier Hayoun <gautier.hayoun@supinfo.com>
+# Copyright (C) 2009 Nicolas Deram <nderam@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +20,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-from decimal import Decimal as decimal
-from re import match
 from copy import deepcopy
+from decimal import Decimal as decimal
+from re import compile
 
 # Import from itools
 from itools.core import freeze
@@ -66,23 +68,19 @@ class Unicode(DataType):
     default = u''
 
 
-    def get_default(cls):
-        return cls.default
-
-
     @staticmethod
     def decode(value, encoding='UTF-8'):
-        return unicode(value, encoding)
+        return unicode(value, encoding).strip()
 
 
     @staticmethod
     def encode(value, encoding='UTF-8'):
-        return value.encode(encoding)
+        return value.strip().encode(encoding)
 
 
     @staticmethod
     def is_empty(value):
-        return value.strip() == u''
+        return value == u''
 
 
 
@@ -134,6 +132,11 @@ class URI(String):
         return True
 
 
+    @staticmethod
+    def is_empty(value):
+        return not value
+
+
 
 class PathDataType(DataType):
     # TODO Do like URI, do not decode (just an string), and use 'is_valid'
@@ -152,12 +155,23 @@ class PathDataType(DataType):
 
 
 
+# We consider the local part in emails is case-insensitive. This is against
+# the standard, but corresponds to common usage.
+email_expr = "^[0-9a-z]+[_\.0-9a-z-'+]*@([0-9a-z-]+\.)+[a-z]{2,6}$"
+email_expr = compile(email_expr)
 class Email(String):
 
+    @staticmethod
+    def encode(value):
+        return value.lower() if value else value
+
+    @staticmethod
+    def decode(value):
+        return value.lower()
+
     @staticmethod
     def is_valid(value):
-        expr = "^[0-9a-z]+[_\.0-9a-z-'+]*@([0-9a-z][0-9a-z-]+\.)+[a-z]{2,4}$"
-        return match(expr, value.lower()) is not None
+        return email_expr.match(value) is not None
 
 
 
@@ -227,11 +241,17 @@ class Enumerate(String):
         return deepcopy(cls.options)
 
 
-    def is_valid(cls, name):
+    def is_valid(self, name):
         """Returns True if the given name is part of this Enumerate's options.
         """
-        options = cls.get_options()
-        return enumerate_is_valid(options, name)
+        options = self.get_options()
+        if isinstance(name, list):
+            options = set([option['name'] for option in options])
+            return set(name).issubset(options)
+        for option in options:
+            if name == option['name']:
+                return True
+        return False
 
 
     def get_namespace(cls, name):
@@ -249,17 +269,8 @@ class Enumerate(String):
         return enumerate_get_value(options, name, default)
 
 
-
-# TODO suppress now there is a single Enumerate class?
-def enumerate_is_valid(options, name):
-    for option in options:
-        if name == option['name']:
-            return True
-    return False
-
-
 def enumerate_get_namespace(options, name):
-    if isinstance(name, list):
+    if type(name) is list:
         for option in options:
             option['selected'] = option['name'] in name
     else:
