@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2007, 2011 J. David Ibáñez <jdavid.ibp@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,16 +15,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from git import describe, get_branch_name, get_filenames, get_metadata
-from git import get_revisions, is_available
+from worktree import open_worktree
 
 
 
-__all__ = [
-    'describe',
-    'get_branch_name',
-    'get_filenames',
-    'get_metadata',
-    'get_revisions',
-    'is_available',
-    ]
+__all__ = ['open_worktree']
