@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2007 Hervé Cauwelier <herve@itaapy.com>
+# Copyright (C) 2004-2010 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2007-2008 Hervé Cauwelier <herve@oursours.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,11 +36,11 @@ from base import Handler
 from config import ConfigFile
 from file import File
 from folder import Folder
-from image import Image
+from image import Image, SVGFile
+from js import JSFile
 from registry import register_handler_class, get_handler_class_by_mimetype
 from text import TextFile, guess_encoding
 from database import RODatabase, RWDatabase
-from database import ROGitDatabase, GitDatabase, make_git_database
 from database import ro_database
 from utils import checkid
 
@@ -53,6 +53,8 @@ __all__ = [
     'File',
     'Folder',
     'Image',
+    'SVGFile',
+    'JSFile',
     'TextFile',
     # Handlers / Archive
     'ZIPFile',
@@ -64,9 +66,6 @@ __all__ = [
     # The database
     'RODatabase',
     'RWDatabase',
-    'ROGitDatabase',
-    'GitDatabase',
-    'make_git_database',
     'ro_database',
     # Registry
     'get_handler_class_by_mimetype',
