@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2006-2007 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2006-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2007 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2006-2007 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2006-2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2007 Sylvain Taverne <taverne.sylvain@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
+from datetime import datetime
 from os.path import join
 from zipfile import ZipFile
 from tarfile import open as open_tarfile
@@ -28,6 +29,19 @@ from file import File
 from registry import register_handler_class
 
 
+
+class Info(object):
+
+    __slots__ = ['name', 'mtime']
+
+    def __init__(self, name, mtime):
+        self.name = name
+        # XXX This datetime is naive because (apparently) neither ZIP nor TAR
+        # archives keep the timezone, so this field is mostly useless
+        self.mtime = mtime
+
+
+
 class ZIPFile(File):
 
     class_mimetypes = ['application/zip']
@@ -39,6 +53,19 @@ class ZIPFile(File):
         return ZipFile(archive)
 
 
+    def get_members(self):
+        zip = self._open_zipfile()
+        try:
+            names = zip.namelist()
+            names.sort()
+            for name in names:
+                member = zip.getinfo(name)
+                mtime = datetime(*(member.date_time))
+                yield Info(name, mtime)
+        finally:
+            zip.close()
+
+
     def get_contents(self):
         zip = self._open_zipfile()
         try:
@@ -60,11 +87,8 @@ class ZIPFile(File):
         try:
             for filename in zip.namelist():
                 path = join(dst, filename)
-                file = lfs.make_file(path)
-                try:
+                with lfs.make_file(path) as file:
                     file.write(zip.read(filename))
-                finally:
-                    file.close()
         finally:
             zip.close()
 
@@ -78,15 +102,32 @@ class TARFile(File):
 
 
     def _open_tarfile(self):
-        name = self.database.fs.get_basename(self.key)
         archive = StringIO(self.to_str())
-        return open_tarfile(name, self.class_mode, fileobj=archive)
+        return open_tarfile(mode=self.class_mode, fileobj=archive)
+
+
+    def get_members(self):
+        tar = self._open_tarfile()
+        try:
+            names = tar.getnames()
+            names.sort()
+            for name in names:
+                member = tar.getmember(name)
+                if member.isdir():
+                    name = '%s/' % name
+                mtime = datetime.utcfromtimestamp(member.mtime)
+                yield Info(name, mtime)
+        finally:
+            tar.close()
 
 
     def get_contents(self):
         tar = self._open_tarfile()
         try:
-            return tar.getnames()
+            names = tar.getnames()
+            # Append trailing slash to directories, as it is with Zip files
+            return [
+                ('%s/' % x) if tar.getmember(x).isdir() else x for x in names ]
         finally:
             tar.close()
 
@@ -141,4 +182,3 @@ class Bzip2File(File):
 # Register
 for cls in [ZIPFile, TARFile, TGZFile, TBZ2File, GzipFile, Bzip2File]:
     register_handler_class(cls)
-
