@@ -1,6 +1,8 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2007 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2008 Hervé Cauwelier <herve@itaapy.com>
+# Copyright (C) 2007, 2011-2012 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2008-2009 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2010 Norman Khine <khinester@aqoon.local>
+# Copyright (C) 2011 Henry Obein <henry.obein@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,16 +17,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+# Import from the Standard Library
+import unicodedata
 
-src = (ur"""ÄÅÁÀÂÃĀäåáàâãāÇçÉÈÊËĒéèêëēÍÌÎÏĪíìîïīÑñÖÓÒÔÕØŌöóòôõøōÜÚÙÛŪüúùûū"""
-       ur"""ÝŸȲýÿȳŽž""")
-dst = (ur"""AAAAAAAaaaaaaaCcEEEEEeeeeeIIIIIiiiiiNnOOOOOOOoooooooUUUUUuuuuu"""
-       ur"""YYYyyyZz""")
+
+
+src = (ur"""ÄÅÁÀÂÃĀäåáàâãāăÇçÉÈÊËĒéèêëēğÍÌÎÏĪíìîïīıļÑñÖÓÒÔÕØŌöóòôõøōőÜÚÙÛŪüúùûū"""
+       ur"""ŞşţÝŸȲýÿȳŽž°«»’""")
+dst = (ur"""AAAAAAAaaaaaaaaCcEEEEEeeeeegIIIIIiiiiiilNnOOOOOOOooooooooUUUUUuuuuu"""
+       ur"""SstYYYyyyZz----""")
 
 transmap = {}
 for i in range(len(src)):
     a, b = src[i], dst[i]
-    transmap[ord(a)] = ord(b)
+    transmap[ord(a)] = b
 transmap[ord(u'æ')] = u'ae'
 transmap[ord(u'Æ')] = u'AE'
 transmap[ord(u'œ')] = u'oe'
@@ -32,24 +38,29 @@ transmap[ord(u'Œ')] = u'OE'
 transmap[ord(u'ß')] = u'ss'
 
 
-def checkid(id):
+def checkid(id, soft=True):
     """Turn a bytestring or unicode into an identifier only composed of
     alphanumerical characters and a limited list of signs.
 
     It only supports Latin-based alphabets.
     """
-    if isinstance(id, str):
+    if type(id) is str:
         id = unicode(id, 'utf8')
 
+    # Normalize unicode
+    id = unicodedata.normalize('NFKC', id)
+
     # Strip diacritics
     id = id.strip().translate(transmap)
 
     # Check for unallowed characters
-    allowed_characters = set([u'.', u'-', u'_', u'@'])
-    id = [ (c.isalnum() or c in allowed_characters) and c or u'-' for c in id ]
+    if soft:
+        allowed_characters = set([u'.', u'-', u'_', u'@'])
+        id = [ x if (x.isalnum() or x in allowed_characters) else u'-'
+               for x in id ]
+        id = u''.join(id)
 
     # Merge hyphens
-    id = u''.join(id)
     id = id.split(u'-')
     id = u'-'.join([x for x in id if x])
     id = id.strip('-')
