@@ -1,5 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2003-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
+# Copyright (C) 2003-2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2008 David Versmisse <versmisse@lil.univ-littoral.fr>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +22,7 @@ from warnings import warn
 
 # Import from itools
 from itools.xml import DOCUMENT_TYPE, START_ELEMENT, END_ELEMENT, COMMENT
-from itools.xml import TEXT, XMLError
+from itools.xml import TEXT, XMLError, DocType
 from xhtml import xhtml_uri
 
 
@@ -32,7 +33,7 @@ from xhtml import xhtml_uri
 #     a SGML format that does not support our DocType parser.
 #     But we must only have (for these HTML pages) a "to_str" function
 #     So, ...
-class DocType_ersatz(object):
+class DocType_ersatz(DocType):
 
     def __init__(self, PubidLiteral=None, SystemLiteral=None, intSubset=None):
         self.PubidLiteral = PubidLiteral
