@@ -1,6 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2008 Nicolas Deram <nicolas@itaapy.com>
+# Copyright (C) 2008 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2008 Nicolas Deram <nderam@gmail.com>
+# Copyright (C) 2008-2010 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2008-2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2010 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2010 Norman Khine <khinester@aqoon.local>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -162,7 +166,10 @@ html_attributes = {
     'autocomplete': String, # <input>
     'pluginspage': String, # <embed>
     'quality': String, # <embed>
+    'autoplay': Boolean, # <embed>
     'wrap': String,
+    # HTML5
+    'controls': String,
     }
 
 
@@ -276,6 +283,7 @@ html_elements = [
     EmptyElement('br', core_attrs),
     Element('button', common_attrs + focus_attrs + ['name', 'value', 'type',
         'disabled']),
+    BlockElement('caption', common_attrs),
     Element('cite', common_attrs),
     Element('code', common_attrs),
     EmptyBlockElement('col', common_attrs + cellhalign_attrs +
@@ -388,12 +396,15 @@ html_elements = [
     # TODO Complete and check against an official description or schema (there
     # is not a DTD in HTML5)
     EmptyBlockElement('canvas', common_attrs + ['width', 'height']),
+    EmptyBlockElement('source', common_attrs + ['src', 'type']),
+    BlockElement('video', common_attrs + ['controls', 'src', 'height',
+        'width']),
     # Vendor specific, not approved by W3C
     # for a talk about <embed> see:
     #   http://alistapart.com/articles/byebyeembed
     # FIXME Check the attribute list for <embed>
     EmptyBlockElement('embed', ['quality', 'pluginspage', 'src', 'type',
-        'width', 'height']),
+        'autoplay', 'width', 'height']),
     ]
 
 
