@@ -1,6 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2008 David Versmisse <david.versmisse@itaapy.com>
 # Copyright (C) 2008 Romain Gauthier <romain.gauthier@itaapy.com>
+# Copyright (C) 2008-2009 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2009-2010 J. David Ibáñez <jdavid.ibp@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@ from itools.core import get_abspath
 from itools.datatypes import String
 from itools.handlers import ro_database
 from itools.relaxng import RelaxNGFile
-from itools.xml import register_namespace, has_namespace, ElementSchema
+from itools.xml import register_namespace, has_namespace, XMLNamespace
 
 
 ###########################################################################
@@ -189,13 +190,6 @@ contexts = [
     ]
 
 
-# XXX These elements appear in openDocuments but they are neither in the doc,
-# nor the relaxNG file!
-unknown_elements = [
-    (style_uri, 'default-page-layout'),
-    (style_uri, 'list-level-label-alignment'),
-    ]
-
 ###########################################################################
 # Make the namespaces
 ###########################################################################
@@ -206,7 +200,7 @@ def duplicate_ns(namespaces, first_uri, second_uri):
     namespaces[second_uri] = ns
 
 # Read the Relax NG schema
-rng_file = get_abspath('OpenDocument-strict-schema-v1.1.rng')
+rng_file = get_abspath('OpenDocument-v1.2-cd05-schema.rng')
 rng_file = ro_database.get_handler(rng_file, RelaxNGFile)
 namespaces = rng_file.get_namespaces()
 
@@ -217,12 +211,6 @@ for uri, element_name in inline_elements:
 for uri, element_name in skip_content_elements:
     element = namespaces[uri].get_element_schema(element_name)
     element.skip_content = True
-for uri, element_name in unknown_elements:
-    elements = namespaces[uri].elements
-    if element_name in elements:
-        raise ValueError, 'element "%s" is defined twice' % element_name
-    elements[element_name] = ElementSchema(element_name,
-                                           default_datatype=String)
 for uri, element_name, context in contexts:
     element = namespaces[uri].get_element_schema(element_name)
     element.context = context
@@ -237,3 +225,10 @@ duplicate_ns(namespaces, svg_uri, svg_uri_2)
 for uri, namespace in namespaces.iteritems():
     if not has_namespace(uri):
         register_namespace(namespace)
+
+# Specific to OpenOffice
+officeooo = XMLNamespace(
+    'http://openoffice.org/2009/office', 'officeooo', [],
+    {'rsid': String, 'paragraph-rsid': String})
+
+register_namespace(officeooo)
