@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2009 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2009 Hervé Cauwelier <herve@itaapy.com>
+# Copyright (C) 2009 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2009, 2012 J. David Ibáñez <jdavid.ibp@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 # Import from itools
 from office import MSWord, MSExcel
+from ooxml import MSWordX, MSExcelX, MSPowerPointX
 from ppt import MSPowerPoint
 from rtf import RTF
 
@@ -27,4 +28,8 @@ __all__ = [
     'MSExcel',
     'MSPowerPoint',
     'RTF',
+    # OOXML
+    'MSWordX',
+    'MSExcelX',
+    'MSPowerPointX',
     ]
