@@ -1,5 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2008 David Versmisse <david.versmisse@itaapy.com>
+# Copyright (C) 2008, 2010 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2009-2010 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2010 Hervé Cauwelier <herve@oursours.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,14 +17,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.handlers import TextFile, register_handler_class
-from itools.xml import XMLParser, XML_DECL, START_ELEMENT, END_ELEMENT, TEXT
-from itools.xml import XMLNamespace, ElementSchema, xmlns_uri
-from itools.xml import register_namespace, has_namespace
 from itools.datatypes import Boolean, Date, DateTime, Decimal, Integer
 from itools.datatypes import QName, String, Time, URI
-from itools.uri import get_reference
 from itools.fs import vfs
+from itools.handlers import TextFile, register_handler_class
+from itools.log import log_warning
+from itools.uri import get_reference
+from itools.xml import XMLNamespace, ElementSchema, xml_uri, xmlns_uri
+from itools.xml import XMLParser, XML_DECL, START_ELEMENT, END_ELEMENT, TEXT
+from itools.xml import register_namespace, has_namespace
 
 
 # Namespace of Relax NG
@@ -419,9 +422,13 @@ def make_namespaces(context):
         # Find the prefix
         for prefix, uri in prefix2uri.iteritems():
             if uri == namespace:
+                result[uri] = XMLNamespace(uri, prefix,
+                                           data['elements'].values(),
+                                           data['free_attributes'], String)
                 break
-        result[uri] = (XMLNamespace(uri, prefix, data['elements'].values(),
-                       data['free_attributes'], String))
+        else:
+            log_warning('relaxng: namespace "%s" not found' % namespace)
+
     return result
 
 
@@ -442,6 +449,7 @@ class RelaxNGFile(TextFile):
                    'elements': [],
                    'references': {},
                    'prefix' : {}}
+        context['prefix']['xml'] = xml_uri
 
         # Parse the file
         read_file(context, self.key, file)
