@@ -1,9 +1,11 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007 David Versmisse <david.versmisse@itaapy.com>
-# Copyright (C) 2007-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2008 Henry Obein <henry@itaapy.com>
-# Copyright (C) 2008 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2007 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2007-2009, 2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2008 Gautier Hayoun <gautier.hayoun@supinfo.com>
+# Copyright (C) 2008 Sylvain Taverne <taverne.sylvain@gmail.com>
+# Copyright (C) 2008-2009 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2010 Hervé Cauwelier <herve@oursours.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -50,7 +52,7 @@ except ImportError:
 import itools
 from itools.core import merge_dicts
 from itools.fs import lfs
-from itools.git import git
+from itools.git import open_worktree
 
 
 # Define list of problems
@@ -460,24 +462,24 @@ def create_graph():
     tmp_directory = '%s/ipkg_quality.git' % mkdtemp()
     call(['git', 'clone',  cwd, tmp_directory])
     chdir(tmp_directory)
+    worktree = open_worktree(tmp_directory)
     # First step: we create a list of statistics
     statistics = {}
-    commit_ids = git.get_revisions()
-    commit_ids.reverse()
-    print 'Script will analyse %s commits.' % len(commit_ids)
-    for commit_id in commit_ids:
+    commits = worktree.git_log(reverse=True)
+    print 'Script will analyse %s commits.' % len(commits)
+    for commit in commits:
         # We move to a given commit
-        call(['git', 'reset', '--hard', commit_id])
+        call(['git', 'reset', '--hard', commit['sha']])
         # Print script evolution
         stdout.write('.')
         stdout.flush()
         # We list files
-        filenames = git.get_filenames()
+        filenames = worktree.get_filenames()
         filenames = [ x for x in filenames if x.endswith('.py') ]
         # We get code quality for this files
         stats, files_db = analyse(filenames, ignore_errors=True)
-        metadata = git.get_metadata()
-        date_time = metadata['committer'][1]
+        metadata = worktree.get_metadata()
+        date_time = metadata['committer_date']
         commit_date = date(date_time.year, date_time.month, date_time.day)
         if commit_date not in statistics:
             statistics[commit_date] = stats
@@ -494,7 +496,7 @@ def create_graph():
     for a_date in dates:
         values.append(statistics[a_date])
     # Base graph informations
-    base_title = '[%s %s]' % (project_name, git.get_branch_name())
+    base_title = '[%s %s]' % (project_name, worktree.get_branch_name())
     # We generate graphs
     chdir(cwd)
     for problem_dict in ['code_length', 'aesthetics_problems',
@@ -597,13 +599,14 @@ if __name__ == '__main__':
     options, args = parser.parse_args()
 
     # Filenames
+    worktree = open_worktree('.', soft=True)
     if args:
         filenames = set([])
         for arg in args:
             filenames = filenames.union(glob(arg))
         filenames = list(filenames)
-    elif git.is_available():
-        filenames = git.get_filenames()
+    elif worktree:
+        filenames = worktree.get_filenames()
         filenames = [ x for x in filenames if x.endswith('.py') ]
     else:
         filenames = []
@@ -624,7 +627,7 @@ if __name__ == '__main__':
     # (1) Export graph
     if options.graph:
         # Check if GIT is available
-        if not git.is_available():
+        if not worktree:
             parser.error(u'Error, your project must be versioned with GIT.')
         # Check if create_graph is available
         if not create_graph_is_available:
