@@ -1,7 +1,8 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2008 Gautier Hayoun <gautier.hayoun@itaapy.com>
-# Copyright (C) 2008 Henry Obein <henry@itaapy.com>
+# Copyright (C) 2005-2009 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2008 Gautier Hayoun <gautier.hayoun@supinfo.com>
+# Copyright (C) 2008 Henry Obein <henry.obein@gmail.com>
+# Copyright (C) 2010 Hervé Cauwelier <herve@oursours.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -140,7 +141,6 @@ def get_test_filenames(test_path, force_download):
             if lfs.exists(compressed_dest) is False:
                 src = join(base_uri, '%s%s' % (name, ext))
                 print 'GET %s file' % src
-                dest = join(test_path, name)
                 if vfs.exists(src) is False:
                     print "%s uri does not exists" % src
                     continue
@@ -205,7 +205,6 @@ def output_result(results, file):
     file_size = rjust(file_size[:9], 9)
     file_string = u'%s  %s' % (filename, file_size)
 
-    parser_output = u''
     # output 1
     parser_name, result = results[0]
     if result is None:
