@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2006 Hervé Cauwelier <herve@itaapy.com>
-# Copyright (C) 2008 David Versmisse <david.versmisse@itaapy.com>
+# Copyright (C) 2004-2011 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2006 Hervé Cauwelier <herve@oursours.net>
+# Copyright (C) 2008 David Versmisse <versmisse@lil.univ-littoral.fr>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from namespaces import is_empty
 from parser import XMLParser, DocType, register_dtd, XMLError, XML_DECL
 from parser import DOCUMENT_TYPE, START_ELEMENT, END_ELEMENT, TEXT, COMMENT
 from parser import PI, CDATA
-from utils import xml_to_text
+from utils import is_xml_stream, xml_to_text
 from xml import Element, stream_to_str, get_element, find_end
 from xml import get_qname, get_attribute_qname, get_end_tag, get_doctype
 
@@ -69,5 +69,6 @@ __all__ = [
     'Element',
     # Functions
     'register_dtd',
+    'is_xml_stream',
     'xml_to_text',
 ]
