@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
-# Copyright (C) 2005-2008 Juan David Ibáñez Palomar <jdavid@itaapy.com>
-# Copyright (C) 2008 David Versmisse <david.versmisse@itaapy.com>
-# Copyright (C) 2008 Sylvain Taverne <sylvain@itaapy.com>
+# Copyright (C) 2005-2008, 2010 J. David Ibáñez <jdavid.ibp@gmail.com>
+# Copyright (C) 2008 David Versmisse <versmisse@lil.univ-littoral.fr>
+# Copyright (C) 2008 Sylvain Taverne <taverne.sylvain@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,11 +33,6 @@ them.
 
 This module provides an API to register namespace uris and handlers,
 and to ask this registry.
-
-It also provides a registry from namespace prefixes to namespace uris.
-While namespace prefixes are local to an XML document, it is sometimes
-useful to refer to a namespace through its prefix. This feature must
-be used carefully, collisions
 """
 
 
@@ -50,23 +45,14 @@ xmlns_uri = 'http://www.w3.org/2000/xmlns/'
 # The registry
 ###########################################################################
 namespaces = {}
-prefixes = {}
 
 
 def register_namespace(namespace, *args):
-    """Associates a namespace handler to a namespace uri. It a prefix is
-    given it also associates that that prefix to the given namespace.
+    """Associates a namespace handler to a namespace uri.
     """
     # Register the URI
     namespaces[namespace.uri] = namespace
 
-    # Register the prefix
-    prefix = namespace.prefix
-    if prefix is not None:
-        if prefix in prefixes:
-            warn('The prefix "%s" is already registered.' % prefix)
-        prefixes[prefix] = namespace.uri
-
     # Register additional URIs
     for uri in args:
         namespaces[uri] = namespace
@@ -91,20 +77,6 @@ def has_namespace(namespace_uri):
     return namespace_uri in namespaces
 
 
-def get_namespace_by_prefix(prefix):
-    """Returns the namespace handler associated to the given prefix. If there
-    is none the default namespace handler is returned, and a warning message
-    is issued.
-    """
-    if prefix in prefixes:
-        namespace_uri = prefixes[prefix]
-        return get_namespace(namespace_uri)
-
-    # Use default
-    warn('Unknown namespace prefix "%s" (using default)' % prefix)
-    return namespaces[None]
-
-
 def get_element_schema(namespace, name):
     return get_namespace(namespace).get_element_schema(name)
 
@@ -155,9 +127,10 @@ class ElementSchema(object):
 class XMLNamespace(object):
 
     def __init__(self, uri, prefix, elements=None, free_attributes=None,
-                 default_datatype=None):
+                 default_datatype=None, default_element=None):
         self.uri = uri
         self.prefix = prefix
+        self.default_element = default_element
         self.default_datatype = default_datatype
         # Elements
         self.elements = {}
@@ -178,10 +151,12 @@ class XMLNamespace(object):
         """Returns a dictionary that defines the schema for the given element.
         """
         element = self.elements.get(name)
-        if element is None:
-            raise XMLError, 'unexpected element "%s"' % name
+        if element is not None:
+            return element
+        if self.default_element is not None:
+            return self.default_element(name)
 
-        return element
+        raise XMLError, 'unexpected element "%s"' % name
 
 
     def get_attr_datatype(self, name):
@@ -196,17 +171,7 @@ class XMLNamespace(object):
 
 
 # The default namespace is used for free elements.
-
-class DefaultNamespace(XMLNamespace):
-    """Default namespace handler for elements and attributes that are not
-    bound to a particular namespace.
-    """
-
-    def get_element_schema(self, name):
-        return ElementSchema(name)
-
-
-default_namespace = DefaultNamespace(None, None)
+default_namespace = XMLNamespace(None, None, default_element=ElementSchema)
 
 
 
